## 简介
| TOPIC | 使用场景 |
| :-- | :-- |
| order.submit | 用户下单、JIT拆单 |
| order.payment | 用户支付完成 |
| order.orderStatus | 用户确认收货、用户取消订单、系统自动取消订单 |
| order.erpstatus | 订单已审核、已备货、已交寄、已妥投、客服取消、ERP系统自动取消 |
| order.createChangeOrder | 前台创建换货单 |
| erp.createChangeOrder | ERP创建换货单 |
| order.updateChangeRefundOrderExpressInfo | 用户更新换货订单快递信息、用户更新退货订单快递信息 |
| erp.updateChangeRefundOrderStatus | ERP退换货状态变更|
| order.createRefundOrder | 前台创建退货单 |
| erp.createRefundOrder | ERP创建退货单 |
| order.changeRefundCancel | 用户取消退货单、用户取消换货单 |
| order.deliveryAddress | 用户修改收货地址 |
| erp.deliveryAddressUpdate | 客服修改收货地址 |

## 前台订单状态同步

### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | order.orderStatus |

> 确认订单
```json
{"orderCode":100000,"status":700}
```
> 用户主动取消
```json
{"orderCode":100000,"status":900}
```
> 系统自动取消
```json
{"orderCode":100000,"status":906}
```

## 订单状态同步

### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | order.erpstatus |

> 已审核
```json
{"order_code":100000,"status":100}
```
> 已备货
```json
{"order_code":100000,"status":200}
```
> 已交寄
```json
{
    "order_code":100000,
    "status":600,
    "data":{
        "express_id": 23,
        "express_number":"25276134639"        
    }
}
```
> 订单妥投
```json
{"order_code":100000,"status":700}
```
> 客服取消
```json
{"order_code":100000,"status":901}
```
> 系统自动取消
```json
{"order_code":100000,"status":906}
```


## 前台创建换货单
### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | order.createChangeOrder |

```json
{
	"id": 1,
	"city": "南京市",
	"area_code": "320115",
	"address": "江宁经济技术开发区苏源大道87号（有货物流中心订单组）",
	"consignee_name": "测试",
	"county": "江宁区",
	"delivery_tpye": 10,
	"email": "",
	"exchange_mode": 10,
	"mobile": "13776693005",
	"order_code": 16062099,
	"init_order_code": 16062099,
	"source_order_code": 16062099,
	"province": "江苏省",
	"receipt_time": "2",
	"remark": "用户换货",
	"uid": 8041518,
	"zip_code": "211100",
	"goods": [
		{
			"exchange_reason": "3",
			"goods_type": "1",
			"last_price": "88.00",
			"new_goods_id": 354687,
			"new_product_skc": 279434,
			"new_product_sku": 892152,
			"product_skc": 279434,
			"product_skn": 51152761,
			"product_sku": 892152,
			"remark": ""
		}
	]
}
```

## ERP创建换货单
### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | erp.createChangeOrder |

> 成功

```json
{
    "code": 200, 
    "message": "ok", 
    "data": {
        "id": 1, 
        "init_order_code": "111", 
        "source_order_code": 1111, 
        "new_order_code": 1111, 
        "exchange_id": 111, 
        "returned_id": 111, 
        "exchange_status": 10, 
        "returned_status": 10
    }
}
```

> 客服驳回

```json
{
    "code":500,
    "message":"error",
    "data":{ 
        "id": 1, 
        "init_order_code": "111", 
        "source_order_code": 1111, 
        "new_order_code": 1111, 
        "exchange_id": 111, 
        "returned_id": 111, 
        "exchange_status": 91,  // 客服驳回
        "returned_status": 91   // 客服驳回
    }
}
```

## 前台更新换货快递信息
### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | order.updateChangeRefundOrderExpressInfo |

```json
{   "id":1,
    "apply_id":40,
    "express_number":"603596395359",
    "express_name":"顺丰",
    "express_id":"23",
    type:"change"       // 表示类型为换货
}
```

## ERP更新换货单状态
### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | erp.updateChangeRefundOrderStatus |
> 审核通过
```json
{   
    "id":1,        // id为erp系统换货单id唯一主键
    "status":10,
    "type":"change"     //"change" 表示换货类型,"refund"表示退货类型
}
```
> 商品入库（仅商品寄回发该状态）
```json
{   
    "id":1,        // id为erp系统换货单id唯一主键
    "status":30,
    "type":"change"     //"change" 表示换货类型,"refund"表示退货类型
}
```
> 上门换货（仅上门换货发该状态）
```json
{   
    "id":1,        // id为erp系统换货单id唯一主键
    "status":40,
    "type":"change"     //"change" 表示换货类型,"refund"表示退货类型
}
```
> 换货完成
```json
{   
    "id":1,        // id为erp系统换货单id唯一主键
    "status":50,
    "type":"change"     //"change" 表示换货类型,"refund"表示退货类型
}
```


## 前台创建退货单
### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | order.createRefundOrder |

```json
{
    "uid": "3342109",
    "alipay_account": "13912008849",
    "alipay_name": "xxxx",
    "area_code": "area_code",
    "bank_card": "6228480605428963215",
    "bank_name": "中国农业银行",
    "city": "南京市",
    "county": "",
    "order_code": "308545172",
    "payee_name": "张璐",
    "province": "江苏省",
    "remark": "尺寸不合适",
    "return_amount_mode": "2",
    "order_returned_id":123,//前台表生成的退货订单主键,其余字段同老接口
    "returned_goods_list": [
        {
            "product_sku": "383669",
            "goods_type": "1",
            "last_price": "334",
            "returned_reason": "1",
            "remark": "尺寸不合适",
            "evidence_images": ""
        }
    ]
}
```


## ERP创建退货单
### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | erp.createRefundOrder |


> 成功

```json
{
    "code": 200, 
    "message": "ok", 
    "data": {   
        "id":1,                     //前台退货id唯一主键
        "returned_id":1,            //returned_id为erp退货id唯一主键
        "returned_status":10,
        "real_returned_amount":100,
        "is_return_coupon":"N",
        "return_yoho_coin":11111
    }
}
```

> 客服驳回

```json
{
    "code":500,
    "message":"error",
    "data":{ 
        "id":1,                      //id为前台退货id唯一主键
        "returned_id":1,            //returned_id为erp退货id唯一主键
        "returned_status":91       //客服驳回
    }
}
```


## 前台更新退货快递信息
### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | order.updateChangeRefundOrderExpressInfo |

```json
{   "order_returned_id":1,      //order_returned_id为前台退货id唯一主键
    "apply_id":40,              //apply_id为erp退货id唯一主键
    "express_number":"603596395359",
    "express_name":"顺丰",
    "express_id":"23"，
    "type":"refund"            // 表示类型为退货
}


```

## ERP更新退货单状态
### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | erp.updateChangeRefundOrderStatus |

```json
{   
    "id":1,        // id 为erp系统退货单id唯一主键
    "status":40,            // 0:提交、10:审核通过、20:商品寄回、30:已入库、40:付款结束、91:客服拒退
    "type":"refund",        // change:表示换货类型、refund:表示退货类型
    "real_returned_amount":100,
    "is_return_coupon":"N",
    "return_yoho_coin":11111        
}

```
## 取消退换货

### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | order.changeRefundCancel |

```json
{"id": 428013,  //erp的退货主键,  或者换货主键 ,根据type传不同的值
"status":900, //表示用户主动取消状态
"type":1  //1表示退货,2表示换货
}   
```


## 用户修改订单收货地址

### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | order.deliveryAddress |

```json
[{
    "address": "靖南大街688-10号2楼",
    "address_id": 61,
    "area_code": 330225,
    "city": "宁波市",
    "district": "象山县",
    "email": "canvas220@sina.com",
    "mobile": "15258109727",
    "order_code": 1607472719,
    "parent_order_code": 123,
    "phone": "057465772167",
    "province": "浙江省",
    "user_name": "朱倩倩",
    "zip_code": 315700
}]
```

## 客服修改订单收货地址

### MQ消息数据结构
| 参数 | 值 |
| :-- | :-- |
| exchanges | amq.topic |
| route key | erp.deliveryAddressUpdate |

```json
[{
    "address": "靖南大街688-10号2楼",
    "address_id": 61,
    "area_code": 330225,
    "city": "宁波市",
    "district": "象山县",
    "email": "canvas220@sina.com",
    "mobile": "15258109727",
    "order_code": 1607472719,
    "parent_order_code": 123,
    "phone": "057465772167",
    "province": "浙江省",
    "user_name": "朱倩倩",
    "zip_code": 315700
}]
```